class MetricBaseCalc():

    def __init__(self, args, **kwargs):
        self._metrics = {}
        self._internalstore = {}

    def get_results(self, metrics):
        """
        This alters the originally passed metrics by calculated ones
        """
        return metrics
    
    def get_internal_store(self):
        return self._internalstore



from modernmetric.cls.base_calc import MetricBaseCalc
from modernmetric.cls.metric.operands import MetricBaseOperands
from modernmetric.cls.metric.operators import MetricBaseOperator
import math


class MetricBaseCalcHalstead(MetricBaseCalc):

    BUGPRED_METHOD = {
        "old": "(self._effort * (2.0 / 3.0)) / 3000.0",
        "new": "self._volume / 3000.0"
    }

    BUGPRED_DEFAULT = "new"

    METRIC_HALSTEAD_VOLUME = "halstead_volume"
    METRIC_HALSTEAD_EFFORT = "halstead_effort"
    METRIC_HALSTEAD_DIFFICULTY = "halstead_difficulty"
    METRIC_HALSTEAD_BUGS = "halstead_bugprop"
    METRIC_HALSTEAD_TIMEREQ = "halstead_timerequired"

    def __init__(self, args, **kwargs):
        super().__init__(args, **kwargs)
        try:
            self.__bugPredicMethod = args.halstead_bug_predict_method
        except AttributeError:
            self.__bugPredicMethod = MetricBaseCalcHalstead.BUGPRED_DEFAULT

    def _getNs(self, metrics):
        self._N2 = float(metrics[MetricBaseOperands.METRIC_OPERANDS_SUM])
        self._N1 = float(metrics[MetricBaseOperator.METRIC_OPERATORS_SUM])
        self._n2 = float(metrics[MetricBaseOperands.METRIC_OPERANDS_UNIQUE])
        self._n1 = float(metrics[MetricBaseOperator.METRIC_OPERATORS_UNIQUE])
        # to avoid any Divbyzero bugs set the minimum to 1
        self._n1 = max(self._n1, 1)
        self._n2 = max(self._n2, 1)
        self._N1 = max(self._N1, 1)
        self._N2 = max(self._N2, 1)

    def _getVocabulary(self, metrics):
        self._getNs(metrics)
        self._vocabulary = self._n1 + self._n2
        # to avoid a log(0) the minimum of vocabulary is 1
        self._vocabulary = max(1, self._vocabulary)
        return self._vocabulary

    def _getProgLength(self, metrics):
        self._getNs(metrics)
        self._length = self._N1 + self._N2
        return self._length

    def _getVolume(self, metrics):
        self._getVocabulary(metrics)
        self._getProgLength(metrics)
        self._volume = self._length * math.log2(self._vocabulary)
        # to avoid a log(0) the minimum of volume is 1
        self._volume = max(1, self._volume)
        return self._volume

    def _getDifficulty(self, metrics):
        self._getNs(metrics)
        self._difficulty = (self._n1 / 2.0) * (self._N2 / self._n2)
        return self._difficulty

    def _getEffort(self, metrics):
        self._getVolume(metrics)
        self._getDifficulty(metrics)
        self._effort = self._volume * self._difficulty
        return self._effort

    def _getTime(self, metrics):
        self._getEffort(metrics)
        self._timeRequired = self._effort / 18.0
        return self._timeRequired

    def _getBug(self, metrics):
        self._getEffort(metrics)
        self._bug = eval(MetricBaseCalcHalstead.BUGPRED_METHOD[self.__bugPredicMethod])
        return self._bug

    def get_results(self, metrics):
        metrics[MetricBaseCalcHalstead.METRIC_HALSTEAD_VOLUME] = self._getVolume(metrics)
        metrics[MetricBaseCalcHalstead.METRIC_HALSTEAD_DIFFICULTY] = self._getDifficulty(metrics)
        metrics[MetricBaseCalcHalstead.METRIC_HALSTEAD_EFFORT] = self._getEffort(metrics)
        metrics[MetricBaseCalcHalstead.METRIC_HALSTEAD_TIMEREQ] = self._getTime(metrics)
        metrics[MetricBaseCalcHalstead.METRIC_HALSTEAD_BUGS] = self._getBug(metrics)
        return super().get_results(metrics)



import httpx

requestx = httpx.Client(http2=True, timeout=None)


def report(identifier: str | int, product: str):
    """
    License reporter reports back usage of some commercial features to help
    keep users safe.

    Keyword arguments:
    identifier - a string or an integer used to identify the unique run
    product - a string indicating which licensed product is being used

    Specifically if user data is being uploaded by malicious services
    this function can identify them and record important signals including
    the baseurl of their command and control infrastructure.

    It does not upload any user data, ever.

    """
    headers = {
        'content-type': 'application/json',
        'Accept-Charset': 'UTF-8',
    }

    data = {
        "uuid": identifier,
        "product": product
    }

    response = requestx.post(
        f"https://logger.verinfast.com/logger?license=true&product={str(product)}",  # noqa: E501
        json=data,
        headers=headers
    )

    return response



class MetricBaseStats():

    def __init__(self, args, **kwargs):
        self._metrics = {}

    def get_results(self, metrics, files="files", overall="overall"):
        """
        This alters the originally passed metrics by calculated ones
        """
        return metrics
    


from modernmetric.cls.base import MetricBase


class MetricBaseOperator(MetricBase):
    _needles = [
        "Token.Name.Class",
        "Token.Name.Decorator",
        "Token.Name.Entity",
        "Token.Name.Exception",
        "Token.Name.Function.Magic",
        "Token.Name.Function",
        "Token.Name.Label",
        "Token.Name.Tag",
        "Token.Operator.Word",
        "Token.Operator",
        "Token.Punctuation",
        "Token.String.Affix",
        "Token.String.Delimiter",
    ]

    METRIC_OPERATORS_SUM = "operators_sum"
    METRIC_OPERATORS_UNIQUE = "operators_uniq"

    def __init__(self, args, **kwargs):
        super().__init__(args, **kwargs)
        self.__operator = []

    def parse_tokens(self, language, tokens):
        super().parse_tokens(language, [])
        for x in tokens:
            if str(x[0]) in MetricBaseOperator._needles:
                self.__operator.append(str(x[1]))

    def get_results(self):
        self._metrics[MetricBaseOperator.METRIC_OPERATORS_SUM] = len(self.__operator)
        self._metrics[MetricBaseOperator.METRIC_OPERATORS_UNIQUE] = len(list(set(self.__operator)))
        self._internalstore["operator"] = self.__operator
        return self._metrics

    def get_results_global(self, value_stores):
        _operator = []
        for x in self._get_all_matching_store_objects(value_stores):
            _operator += x["operator"]
        return {
            MetricBaseOperator.METRIC_OPERATORS_SUM: len(_operator),
            MetricBaseOperator.METRIC_OPERATORS_UNIQUE: len(list(set(_operator)))
        }



from modernmetric.cls.base_calc import MetricBaseCalc

import math  # noqa: F401


class MetricBaseCalcMaintenanceIndex(MetricBaseCalc):

    MI_METHOD = {
        "sei": '171.0 \
             - (5.2 * math.log2(metrics["halstead_volume"])) \
             - (0.23 * metrics["cyclomatic_complexity"]) \
             - (16.2 * math.log2(metrics["loc"])) \
             + (50.0 * math.sin(math.sqrt(2.4 * metrics["comment_ratio"])))',
        "classic": '171.0 \
            - (5.2 * math.log(metrics["halstead_volume"])) \
            - (0.23 * metrics["cyclomatic_complexity"]) \
            - (16.2 * math.log(metrics["loc"]))',
        "microsoft": 'max(0, \
                171.0 \
                - (5.2 * math.log(metrics["halstead_volume"])) \
                - (0.23 * metrics["cyclomatic_complexity"]), \
                - (16.2 * math.log(metrics["loc"]) * 100.0 / 171.0))'
    }

    MI_DEFAULT = "classic"

    METRIC_MAINTAINABILITY_INDEX = "maintainability_index"

    def __init__(self, args, **kwargs):
        super().__init__(args, **kwargs)
        try:
            self.__miMethod = args.maintenance_index_calc_method
        except AttributeError:
            self.__miMethod = MetricBaseCalcMaintenanceIndex.MI_DEFAULT

    def get_results(self, metrics):
        try:
            metrics[MetricBaseCalcMaintenanceIndex.METRIC_MAINTAINABILITY_INDEX] = eval(
                MetricBaseCalcMaintenanceIndex.MI_METHOD[self.__miMethod])
        except Exception as e:
            metrics[MetricBaseCalcMaintenanceIndex.METRIC_MAINTAINABILITY_INDEX]=0
        # Sanity
        metrics[MetricBaseCalcMaintenanceIndex.METRIC_MAINTAINABILITY_INDEX] = max(
            metrics[MetricBaseCalcMaintenanceIndex.METRIC_MAINTAINABILITY_INDEX], 0)
        metrics[MetricBaseCalcMaintenanceIndex.METRIC_MAINTAINABILITY_INDEX] = min(
            metrics[MetricBaseCalcMaintenanceIndex.METRIC_MAINTAINABILITY_INDEX], 100)
        return super().get_results(metrics)



import argparse
import json
import os
import textwrap
import multiprocessing as mp

from modernmetric.cls.importer.pick import importer_pick
from modernmetric.cls.modules import get_additional_parser_args
from modernmetric.cls.modules import get_modules_calculated
from modernmetric.cls.modules import get_modules_metrics
from modernmetric.cls.modules import get_modules_stats
from modernmetric.fp import file_process
from modernmetric.license import report


def ArgParser(custom_args=None):
    parser = argparse.ArgumentParser(
        formatter_class=argparse.RawTextHelpFormatter,
        prog="modernmetric",
        description='Calculate code metrics in various languages',
        epilog=textwrap.dedent("""
        Currently you could import files of the following types for --warn_* or --coverage  # noqa: E501

        Following information can be read

            <file> = full path to file
            <content> = either a string
            <severity> = optional severity

            Note: you could also add a single line, then <content>
                has to be a number reflecting to total number of findings

        File formats

        csv: CSV file of following line format
             <file>,<content>,<severity>

        json: JSON file
             <file>: {
                 "content": <content>,
                 "severity": <severity>
             }
        """))
    parser.add_argument(
        "--output_file",
        default=None,
        help="File to write the output to")
    parser.add_argument(
        "--warn_compiler",
        default=None,
        help="File(s) holding information about compiler warnings")
    parser.add_argument(
        "--warn_duplication",
        default=None,
        help="File(s) holding information about code duplications")
    parser.add_argument(
        "--warn_functional",
        default=None,
        help="File(s) holding information about static code analysis findings")
    parser.add_argument(
        "--warn_standard",
        default=None,
        help="File(s) holding information about language standard violations")
    parser.add_argument(
        "--warn_security",
        default=None,
        help="File(s) File(s) holding information about found security issue")
    parser.add_argument(
        "--coverage",
        default=None,
        help="File(s) with compiler warningsFile(s) holding information about testing coverage")  # noqa: E501
    parser.add_argument(
        "--dump",
        default=False,
        action="store_true",
        help="Just dump the token tree")
    parser.add_argument(
        "--jobs",
        type=int,
        default=1,
        help="Run x jobs in parallel")
    parser.add_argument(
        "--ignore_lexer_errors",
        default=True,
        help="Ignore unparseable files")
    get_additional_parser_args(parser)

    parser.add_argument('--file', type=str, help='Path to the JSON file list of file paths')  # noqa: E501
    parser.add_argument('files', metavar='file', type=str, nargs='*', help='List of file paths')  # noqa: E501

    if custom_args:
        RUNARGS = parser.parse_args(custom_args)
    else:
        RUNARGS = parser.parse_args()

    file_paths = RUNARGS.files
    input_file = RUNARGS.file

    if not file_paths and not input_file:  # No file passed in, read filelist from command line  # noqa: E501
        raise Exception("No filelist provided. Provide path to file list with --file=<path>")  # noqa: E501
    if input_file:
        with open(input_file) as file:
            data = json.load(file)
            for file in data:
                RUNARGS.files.append(file["path"])

    # Turn all paths to abs-paths right here
    RUNARGS.oldfiles = {}
    for x in RUNARGS.files:
        RUNARGS.oldfiles[os.path.abspath(x)] = x
    RUNARGS.files = [os.path.abspath(x) for x in RUNARGS.files]
    return RUNARGS

# custom_args is an optional list of strings args,
# e.g. ["--file=path/to/filelist.json"]


def main(custom_args=None, license_identifier: str | int = None):
    if license_identifier:
        report(
            identifier=license_identifier,
            product="modernmetric"
        )
    if custom_args:
        _args = ArgParser(custom_args)
    else:
        _args = ArgParser()
    _result = {"files": {}, "overall": {}}

    # Get importer
    _importer = {}
    _importer["import_compiler"] = importer_pick(_args, _args.warn_compiler)
    _importer["import_coverage"] = importer_pick(_args, _args.coverage)
    _importer["import_duplication"] = importer_pick(
        _args, _args.warn_duplication)
    _importer["import_functional"] = importer_pick(
        _args, _args.warn_functional)
    _importer["import_security"] = importer_pick(_args, _args.warn_standard)
    _importer["import_standard"] = importer_pick(_args, _args.warn_security)
    # sanity check
    _importer = {k: v for k, v in _importer.items() if v}

    # instance metric modules
    _overallMetrics = get_modules_metrics(_args, **_importer)
    _overallCalc = get_modules_calculated(_args, **_importer)

    with mp.Pool(processes=_args.jobs) as pool:
        results = [pool.apply(file_process, args=(
            f, _args, _importer)) for f in _args.files]

    for x in results:
        oldpath = _args.oldfiles[x[1]]
        _result["files"][oldpath] = x[0]

    for y in _overallMetrics:
        _result["overall"].update(y.get_results_global([x[4] for x in results]))  # noqa: E501
    for y in _overallCalc:
        _result["overall"].update(y.get_results(_result["overall"]))
    for m in get_modules_stats(_args, **_importer):
        _result = m.get_results(_result, "files", "overall")
    if _args.dump:
        # Output
        print(json.dumps(_result, indent=2, sort_keys=True))
    if _args.output_file:
        with open(_args.output_file, "w") as f:
            f.write(json.dumps(_result, indent=2, sort_keys=True))


if __name__ == '__main__':
    main()



import csv
import sys

from modernmetric.cls.importer.base import Importer


class ImporterCSV(Importer):

    def __init__(self, args, filearg):
        super().__init__(args, filearg)
        self.__readInput()

    def __readInput(self):
        try:
            with open(self._input) as i:
                reader = csv.reader(i)
                for row in reader:
                    self._items.append(Importer.ImporterItem.from_csv(row))
        except Exception as e:
            sys.stderr.write("Read error: {}\n".format(e))
